# to require all the related gems and functionalities to this spec we need to give this in every spec file.
require 'spec_helper'

# Give a unique name in describe so that allure report will be created seperately. 
# Note: If two spec has same name, allure will override the results of first spec with the second spec which has same name.
describe "Test Spec" do
  
  before(:all) do
    # Initialize all the constants here.

    # Initialize a driver with a name, so that when there is a bug screen shots will be saved with the name.
    # Multiple drivers can be initialized (with different properties, check driver.rb file in the gem for details)
    # We can initialize one chrome, one firefox, one explorer, one safari browser at a time.
    @driver1 = Driver.new("Test_Driver_1")
    @driver2 = Driver.new("Test_Driver_2")
    @driver3 = Driver.new("Test_Driver_3")
    @driver4 = Driver.new("Test_Driver_4")

    # To load the constants file from Constants class.
    @constants = Constants.new

    # Initialize the page class with the driver.
    # Each instance method will be corresponding to the driver with which it is declared.
    # New pages can be added and used in the same way
    @driver1_login_page = Pages::TestPage.new(@driver1)
    @driver2_login_page = Pages::TestPage.new(@driver2)
    @driver3_login_page = Pages::TestPage.new(@driver3)
    @driver4_login_page = Pages::TestPage.new(@driver4)

  end

  before(:each) do
    # any specfic steps that needs to be done before each testcase can be done here
    # Example : Moving to the home page after all the test case will be an idle one.
  end

  after(:all) do
    # Steps that is to be run after all the test cases are done can be given here.
    # Usual steps will be quiting all the drivers, but that is handled in the spec_helper.rb so any other specific steps other than quit can be given here.
  end

  after(:each) do |e|
    # any specfic steps that needs to be done after each testcase can be done here
    # Usually taking screen shots after each test case when there is a failure is done here, but that is covered in spec_helper.rb
    # Any other steps than taking screenshot can be given here.
  end


  # Each test case can be declared with `it` 
  # after the test case name is defined we can define the tags for that test case
  # These tags can be used to run only critical test cases, only sanity test cases or we can use the testId for any custom reporting.
  it 'this test will pass',:severity => "critical", :sanity => true, :testId => '001' do |e|
    # for more help on rspec expect statements you can visit the following website
    # https://relishapp.com/rspec/rspec-expectations/docs/built-in-matchers
    
    # e.step is used by allure for reporting purpose. It is best to have all the lines inside any one of the e.step so that we can have a detailed reporting
    e.step "true eql true" do
      expect(true).to eq true
    end
    e.step "false eql false" do
      expect(false).to eq false
    end
    e.step "string eql string" do
      expect("test").to eq "test"
    end
    e.step "number eql number" do
      expect(1).to eq 1
    end
  end

  it "this test will fail", :severity => "critical", :testId => '002' do |e|
    e.step "true eql true" do
      expect(true).to eq true
    end
    e.step "false eql false" do
      expect(false).to eq false
    end
    # Test case will fail in the below step
    e.step "true eql false" do
      expect(true).to eq false
    end
    # Since the above step is failed, following steps will not be executed.
    e.step "true eql true" do
      expect(true).to eq true
    end
    e.step "false eql false" do
      expect(false).to eq false
    end
  end

  it "this test will fail", :severity => "critical", :testId => '003' do |e|
    e.step "true eql true" do
      expect(true).to eq true
    end
    e.step "false eql false" do
      expect(false).to eq false
    end
    # Test case will fail in the below step
    # cstep is the custom method in which if a step fails it wont stop the test case
    # this cstep can be used to validate data in a page, if we need to test and report 10 variables in a page, then we can use this so that all the validation point steps even if one step is failed
    e.cstep "1 equal 2" do
      expect(1).to eq 2
    end
    # even if the  the above step is failed, following steps will be executed.
    e.step "true eql true" do
      expect(true).to eq true
    end
    e.step "false eql false" do
      expect(false).to eq false
    end
  end


  it 'Calling a function in a `Page` and `Object` from the spec file',:severity => "critical", :testId => '004' do |e|
    # we can call a funtion in the pages in the following ways
    e.step "Login to the application" do
      @driver1_login_page.login(@constants.COMMON_EMAIL, @constants.COMMON_PASSWORD, @constants.BASE_URL)
      Pages::TestPage.new(@driver2).login(@constants.COMMON_EMAIL, @constants.COMMON_PASSWORD, @constants.BASE_URL)
    end
    
    # we can call a locator with the page instance itself as follows,
    e.step "Is login link is present" do
      expect(@driver1_login_page.LOGIN_LINK.is_present?).to eql false
      expect(@driver1_login_page.LOGIN_DYNAMIC("Login").is_present?).to eql false
    end

  end


  
end
